eltype_or(::Type{G}, or) where G <: (AbstractGeometry{N, T} where N) where T = T
eltype_or(::Type{G}, or) where G <: (AbstractGeometry{N, T} where {N, T}) = or

ndims_or(::Type{G}, or) where G <: (AbstractGeometry{N, T} where T) where N = N
ndims_or(::Type{G}, or) where G <: (AbstractGeometry{N, T} where {N, T}) = or


Base.eltype(T::Type{<:AbstractGeometry}) = eltype_or(T, Any)

function Base.ndims(T::Type{<:AbstractGeometry})
    ndims_or(T, Any)
end

Base.eltype(x::AbstractGeometry{N, T}) where {N, T} = T
Base.ndims(x::AbstractGeometry{N, T}) where {N, T} = N

Base.eltype(::Type{AFG{T}}) where {T} = T
Base.eltype(::Type{FG}) where {FG <: AFG} = eltype(supertype(FG))





using Base.Cartesian

"""
deleteat{N,T,i}(x::NTuple{N,T}, ::Type{Val{i}})

Return copy of x with ith entry ommited.
"""
@generated function deleteat(x::NTuple{N,T}, ::Type{Val{i}}) where {N,T,i}
    (1 <= i <= N) || throw(MethodError(drop_index, (x,Val{i})))
    args = [:(x[$j]) for j in deleteat!([1:N...], i)]
    Expr(:tuple, args...)
end

"""
deleteat{N,T}(x::NTuple{N,T}, i::Int)
"""
@generated deleteat(x::NTuple{N,T}, i::Int) where {N,T} = quote
    # would be nice to eliminate boundscheck
    (1 <= i <= N) || throw(BoundsError(x,i))
    @nif $(N) d->(i==d) d-> deleteat(x, Val{d})
end

"""
best_arg, best_val = argmax(f, iter)

Computes the first element of iterator, on which f takes its maximum.
"""
function argmax(f, iter)
    iter_result = iterate(iter)
    if iter_result === nothing
        throw(ArgumentError("Cannot compute the argmax of an empty collection"))
    end
    best_arg, state = iter_result
    best_val = f(best_arg)
    while true
        iter_result = iterate(iter, state)
        iter_result === nothing && break
        arg, state = iter_result
        val = f(arg)
        if val > best_val
            best_val = val
            best_arg = arg
        end
    end
    best_arg, best_val
end

w_component(::Type{Point{N, T}}) where {N, T} = T(1)
w_component(::Type{Vec{N, T}}) where {N, T} = T(0)

@generated function transform_convert(::Type{T1}, x::T2) where {T1 <: StaticVector, T2 <: StaticVector}
    w = w_component(T1)
    n1 = length(T1)
    n2 = length(T2)
    n1 <= n2 && return :(T1(x))
    tupl = Expr(:tuple)
    ET = eltype(T1)
    for i = 1:n2
        push!(tupl.args, :($ET(x[$i])))
    end
    for i = 1:(n1 - n2 - 1)
        push!(tupl.args, :($ET(0)))
    end
    push!(tupl.args, :($w))
    :(T1($tupl))
end


@inline sqnorm(x) = sum(x.^2)

"""
pinvli{n,m,T}(a::Mat{n,m,T})

Compute pseudo inverse of matrix with linear independent columns.
"""
function pinvli(a::Mat{n,m,T}) where {n,m,T}
    @assert n >= m
    return inv(a'*a)*a'
end
